/*
 * Telapathy Inspector - A Telepathy client which exposes Telepathy interfaces.
 *                       Meant to inspect and/or test connection managers.
 * 
 * ti-dlg-request-aliases.c:
 * Dialog for Connection.Interface.Aliasing.RequestAliases()
 * 
 * Copyright (C) 2006 INdT - Instituto Nokia de Tecnologia
 * Author - Daniel d'Andrada T. de Carvalho <daniel.carvalho@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "ti-dlg-request-aliases.h"
#include "ti-util.h"
#include "ti-config.h"
#include "ti-handle-list-editor.h"

#include <glade/glade.h>

G_DEFINE_TYPE (TIDlgRequestAliases, ti_dlg_request_aliases, G_TYPE_OBJECT);

/**
 * Instance private data.
 */
struct _TIDlgRequestAliasesPrivate {
    GtkWindow* parent;

    GladeXML* glade_xml;

    TIHandleListEditor* handle_list_editor;

    GtkWidget* dialog;
};
typedef struct _TIDlgRequestAliasesPrivate TIDlgRequestAliasesPrivate;

#define TI_DLG_REQUEST_ALIASES_GET_PRIVATE(object)  (G_TYPE_INSTANCE_GET_PRIVATE ((object), TI_TYPE_DLG_REQUEST_ALIASES, TIDlgRequestAliasesPrivate))

/* Function prototypes */
static void _button_ok_clicked (GtkButton *button, gpointer user_data);
static void _button_cancel_clicked (GtkButton *button, gpointer user_data);
static void _ti_dlg_request_aliases_setup (TIDlgRequestAliases* dlg_request_aliases);

/**
 * Drop all references to other objects.
 */
static void
ti_dlg_request_aliases_dispose (GObject *object)
{
    TIDlgRequestAliases *dlg_request_aliases = TI_DLG_REQUEST_ALIASES (object);
    TIDlgRequestAliasesPrivate *priv = TI_DLG_REQUEST_ALIASES_GET_PRIVATE (dlg_request_aliases);

    if (priv->glade_xml != NULL) {
        g_object_unref (priv->glade_xml);
        priv->glade_xml = NULL;
    }

    G_OBJECT_CLASS (ti_dlg_request_aliases_parent_class)->dispose (object);
}

/**
 * Class initialization.
 */
static void
ti_dlg_request_aliases_class_init (TIDlgRequestAliasesClass *ti_dlg_request_aliases_class)
{
	GObjectClass *gobject_class = G_OBJECT_CLASS (ti_dlg_request_aliases_class);

	/* override base object methods */ 
	gobject_class->dispose = ti_dlg_request_aliases_dispose;
		
	/* Add private */
	g_type_class_add_private (ti_dlg_request_aliases_class, sizeof (TIDlgRequestAliasesPrivate));
}

/**
 * Instance initialization.
 */
static void
ti_dlg_request_aliases_init (TIDlgRequestAliases *ti_dlg_request_aliases)
{
//    TIDlgRequestAliasesPrivate* priv = TI_DLG_REQUEST_ALIASES_GET_PRIVATE (ti_dlg_request_aliases);
}

/**
 * Returns a new instance.
 */
TIDlgRequestAliases*
ti_dlg_request_aliases_new (GtkWindow* parent, TIHandleMapper* handle_mapper) 
{
    TIDlgRequestAliases* dialog = NULL;
    TIDlgRequestAliasesPrivate* priv = NULL;
    gchar* glade_file_path = NULL;

	dialog = g_object_new (TI_TYPE_DLG_REQUEST_ALIASES, NULL);

    priv = TI_DLG_REQUEST_ALIASES_GET_PRIVATE (dialog);
    priv->parent = parent;
    priv->handle_list_editor = TI_HANDLE_LIST_EDITOR (ti_handle_list_editor_new (handle_mapper));

    glade_file_path = g_strdup_printf ("%s%s", TI_DATA_DIR_PREFIX, "dlg-request-aliases.xml");
    priv->glade_xml = glade_xml_new (glade_file_path, NULL, NULL);
    if (priv->glade_xml == NULL)
    {
        g_critical ("Error loading glade file \"%s\".", glade_file_path);
        g_object_unref (dialog);
        dialog = NULL;
        goto CLEAN_UP;
    }

    _ti_dlg_request_aliases_setup (dialog);

    gtk_window_set_transient_for (GTK_WINDOW (priv->dialog), GTK_WINDOW (priv->parent));
    gtk_window_set_position (GTK_WINDOW (priv->dialog), GTK_WIN_POS_CENTER_ON_PARENT);

    glade_xml_signal_connect_data (priv->glade_xml, "button_ok_clicked", G_CALLBACK(_button_ok_clicked), priv->dialog);
    glade_xml_signal_connect_data (priv->glade_xml, "button_cancel_clicked", G_CALLBACK(_button_cancel_clicked), priv->dialog);

    CLEAN_UP:
    g_free (glade_file_path);

    return dialog;
}

/**
 * Run
 */
gboolean
ti_dlg_request_aliases_run (TIDlgRequestAliases* dlg_request_aliases, GArray** handles)
{
    TIDlgRequestAliasesPrivate *priv = TI_DLG_REQUEST_ALIASES_GET_PRIVATE (dlg_request_aliases);
    gint result;

    g_assert (handles != NULL);

    ti_handle_list_editor_set (priv->handle_list_editor, NULL);

    gtk_widget_show_all (priv->dialog);
    result = gtk_dialog_run (GTK_DIALOG (priv->dialog));
    gtk_widget_hide_all (priv->dialog);

    *handles = ti_handle_list_editor_get (priv->handle_list_editor);

    if (*handles == NULL)
    {
        return FALSE;
    }
    else
    {
        return result == GTK_RESPONSE_OK;
    }
}

/**
 * Called when OK button is clicked.
 */
static void
_button_ok_clicked (GtkButton *button, gpointer user_data)
{
    GtkDialog* dialog = GTK_DIALOG (user_data);
    gtk_dialog_response (dialog, GTK_RESPONSE_OK);
}

/**
 * Called when "Cancel" button is clicked.
 */
static void
_button_cancel_clicked (GtkButton *button, gpointer user_data)
{
    GtkDialog* dialog = GTK_DIALOG (user_data);
    gtk_dialog_response (dialog, GTK_RESPONSE_CANCEL);
}

/**
 * Setup
 */
static void
_ti_dlg_request_aliases_setup (TIDlgRequestAliases* dlg_request_aliases)
{
    TIDlgRequestAliasesPrivate *priv = TI_DLG_REQUEST_ALIASES_GET_PRIVATE (dlg_request_aliases);
    GtkWidget* widget;

    priv->dialog = glade_xml_get_widget (priv->glade_xml, "dialog_request_aliases");
    g_assert (GTK_IS_DIALOG (priv->dialog));

    // Handle List Editor
    widget = glade_xml_get_widget (priv->glade_xml, "container_handle_list_editor");
    g_assert (widget != NULL && GTK_IS_CONTAINER (widget));
    gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (priv->handle_list_editor));
}
